==> pymetrick-0.41.zip: pymetrick-0.41/pymetrick/routing.py <==
#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Modulo de gestion de rutas"""

__author__ = 'Fco.Javier Tamarit V <javtamvi@gmail.com>'
__copyright__ = "Copyright (C) 2012 PYMETRICK.ORG"
__date__ = '2012-09-21'
__version__ = '0.4'
__license__ = 'Licencia GNU/GPLv2'
__credits__ = ''
__text__ = 'Gestion de rutas'
__file__ = 'routing.py'

#--- CHANGES ------------------------------------------------------------------
# 2012-09-21 v0.01 PL: - First version


"""
    He aquí un ejemplo simple que crea un mapa de URL para una aplicación con
    dos subdominios (www y kb) y algunas reglas URL:

    >>> m = Map([
    ...     # Static URLs
    ...     ('/', endpoint='static/index'),
    ...     ('/about', endpoint='static/about'),
    ...     ('/help', endpoint='static/hcd
    ...     Subdomain('kb', [
    ...         Rule('/', endpoint='kb/index'),
    ...         Rule('/browse/', endpoint='kb/browse'),
    ...         Rule('/browse/<int:id>/', endpoint='kb/browse'),
    ...         Rule('/browse/<int:id>/<int:page>', endpoint='kb/browse')
    ...     ])
    ... ], default_subdomain='www')



"""
import sys
import os
import re
import logging

# Version PY
PY2 = True if sys.version[:1] == 2 else False;
PY3 = True if sys.version[:1] == 3 else False;

# En Python 3  long = int
if PY3:
    long = int

# Comprobar codigo en desarrollo
DEBUG = False

class Map(object):
    """Map relaciona las rutas con los controladores a ejecutar.
    existen dos parametros esenciales :
    default_subdomain : Dominio por defecto www
    redirect_default  : Cuando no existe la ruta solicitada, se redirige a una ruta por defecto.
    url_map = Map([
                ('/', subdomain='{username}', endpoint='user/homepage'),
                ('/stats', subdomain='{username}', endpoint='user/stats')
            ])
    Para incorporar nuevas rutas utiliza add, si una ruta ya existiera, se sustituira por la incorporada.
    add('/ideas/{idea}', controller = 'idea')
    add('/users/{usuario}', controller = 'user')
    add('/tags/{etiquetas}', controller = 'tag')

    _rules  es el conjunto de rutas de validacion, cada elemento de _rules se compone de una ruta y un controlador
            Cada ruta puede ser estatica p.e. : /login o dinámica p.e. : /articulos/{año}/{dia}
    """

    #Tratamiento de errores en LOG
    __name__ = "routing.Map"
    sys.stdout = sys.stderr

    if DEBUG:
        logPath = ''
        logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
        LOG = logging.getLogger(__name__)
        hdlr = logging.FileHandler(logPath+__name__+'.log')
        formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
        hdlr.setFormatter(formatter)
        LOG.addHandler(hdlr)
        LOG.info("Session modo local INICIO")
    else:
        logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
        LOG = logging.getLogger(__name__)

    def __init__(self, default_subdomain='', redirect_defaults=True):

        self._rules = {}
        self.default_subdomain = default_subdomain
        self.redirect_defaults = redirect_defaults


    def add(self, rule='', controller=''):
        """Incorporar una nueva ruta dirigiendo la accion a un controlador
        _rules  reglas de validacion

        """
        try:
            if rule and controller:
                """Incorporar nueva regla o sustituir si existe"""
                self._rules[rule]=controller
        except Exception,e:
            tb = sys.exc_info()[2]
            self.LOG.error('Error en add <%s> en linea %s !!!' % (str(e),tb.tb_lineno))

    def match(self, path):
        """Compruebe si la regla coincide con una ruta determinada"""
        _match = []
        _match_number = 0
        # Comprobar si la ruta contiene datos del tipo ?nombre_variable=dato y extrae solo informacion de ruta
        if re.search(r'\?',path):
            path = path[:path.find(r"?")]
        # Si es un path dinamico debemos comprobarlo minuciosamente
        _dict_dinamico = dict()
        try:

            _path = path.strip("/").split("/")
            for n,m in self._rules.iteritems():
                _n = n.strip("/").split("/")
                _match_number = 0
                for i in xrange(0,len(_n)):
                    if len(_path)>=i+1:
                        if _n[i].startswith('{') and _n[i].endswith('}') and len(_path)>=i+1:
                            # Los parametros en la ruta aparecen entre {}
                            # crear un diccionario emparejando parametros con valor
                            _dict_dinamico[_n[i].strip("{}")]=_path[i]
                            _match_number +=1
                        elif _path[i]==_n[i]:
                            # Cuenta coincidencias entre ruta y valor
                            _match_number += 1
                        if len(_path)==_match_number and _match_number>0:
                            if len(_path)==_match_number and len(_n)==_match_number:
                                # Prepara resultado : ruta, controlador, coincidencia
                                if len(_dict_dinamico)>0:
                                    # Crear la funcion con parametros
                                    m = "%s(**%s)" % (m,_dict_dinamico)
                                _match.append((n,m,_match_number))

            if not _match and self.redirect_defaults:
                """404	NOT FOUND. El servidor no halló nada en la dirección especificada"""
                """Devuelve resultado como ruta, controlador y coincidencia"""
                _match.append(('/','404',0))
            """Devuelve solo la funcion a evaluar"""
            return _match[0][1]
        except Exception,e:
            tb = sys.exc_info()[2]
            self.LOG.error('Error en Map.match <%s> en linea %s !!!' % (str(e),tb.tb_lineno))

    def __del__(self):
        '''Eliminar objeto y objetos dependientes'''
        if DEBUG:
            self.LOG.debug("del ha eliminado el objeto")
        else:
            pass

if __name__ == "__main__":
    if len(sys.argv) < 2:
        sys.stderr.write("For help use -h o --help")
    elif sys.argv[1]=='-h' or sys.argv[1]=='--help':
        print (u'''
        Primero debe crear una ruta al subdominio de la siguiente forma :

        rules_web = Map(default_subdomain='www',redirect_defaults='/')

        Las rutas a páginas se crean de la siguiente forma :

        rules_web.add(rule='/login',controller='login')
        rules_web.add(rule='/login/nada',controller='login')
        rules_web.add(rule='/login/nada/todo',controller='login')
        rules_web.add(rule='/registro',controller='registro')
        rules_web.add(rule='/prueba',controller='prueba')

        Ahora, cuando busque una ruta debe indicarlo como :

        rules_web.match('login')
        rules_web.match('/login/nada/todo')
        rules_web.match('/varios')

        esto ejecutara el controlador parametrizado.

        Si la ruta obtenida del cliente, contiene datos del tipo
        '/auth?user=javier&passw=tonto'
        se eliminaran para evaluar correctamente la ruta

        Y si no existe la ruta, devolvera 404 y podremos redirigirlo
        if rules_web.match('/no_existe')=="404":
           rules_web.match('/')
        ''')

==> pymetrick-0.42.zip: pymetrick-0.42/pymetrick/routing.py <==
#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Modulo de gestion de rutas"""

from pymetrick import version
__author__ = version.__author__
__copyright__ = version.__copyright__
__license__ = version.__license__
__version__ = version.__version__
__date__ = '2012-09-21'
__credits__ = ''
__text__ = 'Gestion de rutas'
__file__ = 'routing.py'

#--- CHANGES ------------------------------------------------------------------
# 2012-09-21 v0.01 PL: - First version


"""
    He aquí un ejemplo simple que crea un mapa de URL para una aplicación con
    dos subdominios (www y kb) y algunas reglas URL:

    >>> m = Map([
    ...     # Static URLs
    ...     ('/', endpoint='static/index'),
    ...     ('/about', endpoint='static/about'),
    ...     ('/help', endpoint='static/hcd
    ...     Subdomain('kb', [
    ...         Rule('/', endpoint='kb/index'),
    ...         Rule('/browse/', endpoint='kb/browse'),
    ...         Rule('/browse/<int:id>/', endpoint='kb/browse'),
    ...         Rule('/browse/<int:id>/<int:page>', endpoint='kb/browse')
    ...     ])
    ... ], default_subdomain='www')



"""
import sys
import os
import re
import logging

# Version PY
PY2 = True if sys.version[:1] == 2 else False;
PY3 = True if sys.version[:1] == 3 else False;

# En Python 3  long = int
if PY3:
    long = int

# Comprobar codigo en desarrollo
DEBUG = False

class Map(object):
    """Map relaciona las rutas con los controladores a ejecutar.
    existen dos parametros esenciales :
    default_subdomain : Dominio por defecto www
    redirect_default  : Cuando no existe la ruta solicitada, se redirige a una ruta por defecto.
    url_map = Map([
                ('/', subdomain='{username}', endpoint='user/homepage'),
                ('/stats', subdomain='{username}', endpoint='user/stats')
            ])
    Para incorporar nuevas rutas utiliza add, si una ruta ya existiera, se sustituira por la incorporada.
    add('/ideas/{idea}', controller = 'idea')
    add('/users/{usuario}', controller = 'user')
    add('/tags/{etiquetas}', controller = 'tag')

    _rules  es el conjunto de rutas de validacion, cada elemento de _rules se compone de una ruta y un controlador
            Cada ruta puede ser estatica p.e. : /login o dinámica p.e. : /articulos/{año}/{dia}
    """

    #Tratamiento de errores en LOG
    __name__ = "routing.Map"
    sys.stdout = sys.stderr

    if DEBUG:
        logPath = ''
        logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
        LOG = logging.getLogger(__name__)
        hdlr = logging.FileHandler(logPath+os.sep+__name__+'.log')
        formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
        hdlr.setFormatter(formatter)
        LOG.addHandler(hdlr)
        LOG.info("Routing modo local INICIO")
    else:
        logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
        LOG = logging.getLogger(__name__)

    def __init__(self, default_subdomain='', redirect_defaults=True):

        self._rules = {}
        self.default_subdomain = default_subdomain
        self.redirect_defaults = redirect_defaults


    def add(self, rule='', controller=''):
        """Incorporar una nueva ruta dirigiendo la accion a un controlador
        _rules  reglas de validacion

        """
        try:
            if rule and controller:
                """Incorporar nueva regla o sustituir si existe"""
                self._rules[rule]=controller
        except Exception as e:
            tb = sys.exc_info()[2]
            self.LOG.error('Error en add <%s> en linea %s !!!' % (str(e),tb.tb_lineno))

    def match(self, path):
        """Compruebe si la regla coincide con una ruta determinada"""
        _match = []
        _match_number = 0
        # Comprobar si la ruta contiene datos del tipo ?nombre_variable=dato y extrae solo informacion de ruta
        if re.search(r'\?',path):
            path = path[:path.find(r"?")]
        # Si es un path dinamico debemos comprobarlo minuciosamente
        _dict_dinamico = dict()
        try:

            _path = path.strip("/").split("/")
            for n,m in self._rules.iteritems():
                _n = n.strip("/").split("/")
                _match_number = 0
                for i in xrange(0,len(_n)):
                    if len(_path)>=i+1:
                        if _n[i].startswith('{') and _n[i].endswith('}') and len(_path)>=i+1:
                            # Los parametros en la ruta aparecen entre {}
                            # crear un diccionario emparejando parametros con valor
                            _dict_dinamico[_n[i].strip("{}")]=_path[i]
                            _match_number +=1
                        elif _path[i]==_n[i]:
                            # Cuenta coincidencias entre ruta y valor
                            _match_number += 1
                        if len(_path)==_match_number and _match_number>0:
                            if len(_path)==_match_number and len(_n)==_match_number:
                                # Prepara resultado : ruta, controlador, coincidencia
                                if len(_dict_dinamico)>0:
                                    # Crear la funcion con parametros
                                    m = "%s(**%s)" % (m,_dict_dinamico)
                                _match.append((n,m,_match_number))

            if not _match and self.redirect_defaults:
                """404  NOT FOUND. El servidor no halló nada en la dirección especificada"""
                """Devuelve resultado como ruta, controlador y coincidencia"""
                _match.append(('/','404',0))
            """Devuelve solo la funcion a evaluar"""
            return _match[0][1]
        except Exception as e:
            tb = sys.exc_info()[2]
            self.LOG.error('Error en Map.match <%s> en linea %s !!!' % (str(e),tb.tb_lineno))

    def __del__(self):
        '''Eliminar objeto y objetos dependientes'''
        if DEBUG:
            self.LOG.debug("del ha eliminado el objeto")
        else:
            pass

if __name__ == "__main__":
    if len(sys.argv) < 2:
        sys.stderr.write("For help use -h o --help")
    elif sys.argv[1]=='-h' or sys.argv[1]=='--help':
        print (u'''version {0}'''.format( __version__))
        print (u'''
        Primero debe crear una ruta al subdominio de la siguiente forma :

        rules_web = Map(default_subdomain='www',redirect_defaults='/')

        Las rutas a páginas se crean de la siguiente forma :

        rules_web.add(rule='/login',controller='login')
        rules_web.add(rule='/login/nada',controller='login')
        rules_web.add(rule='/login/nada/todo',controller='login')
        rules_web.add(rule='/registro',controller='registro')
        rules_web.add(rule='/prueba',controller='prueba')

        Ahora, cuando busque una ruta debe indicarlo como :

        rules_web.match('login')
        rules_web.match('/login/nada/todo')
        rules_web.match('/varios')

        esto ejecutara el controlador parametrizado.

        Si la ruta obtenida del cliente, contiene datos del tipo
        '/auth?user=javier&passw=tonto'
        se eliminaran para evaluar correctamente la ruta

        Y si no existe la ruta, devolvera 404 y podremos redirigirlo
        if rules_web.match('/no_existe')=="404":
           rules_web.match('/')
        ''')
